set more off

* Set number of reps for bootstrap randomization
local numberofreps=100000

use processeddata\Final_Anonymized_Analysis_Dataset.dta
rename outcome_felonrecord outcome_supervisionrecord

count
if r(N)<`numberofreps' {
 set obs `numberofreps'
}
gen rep_number=_n
replace rep_number=. if rep_number>`numberofreps'
gen simulated_beta=.

*Merge in past simulation results. As a note, this is to deal with crashes . Because we are setting a randomization seed, we still need to do the randomizations below for those earlier iterations, but we don't need to do the sorting/regression/etc.
capture joinby rep_number using intermediate_randomization_test_results, update unmatched(master)
capture drop observed_beta  observed_inflationfactor
qui summ rep_number if simulated_beta~=.
local completedsims=0
if r(max)~=. {
 local completedsims=r(max)
}
di "Already completed this many simulations: `completedsims'"

*about 90% of records are assigned to treatment in each state
tab match_state treatment, row

*For each state, count the number of people in it. This will speed up assigning 90% to treatment below
bysort match_state: gen num_in_state=_N
set seed 1238123

*regression used in paper
regress outcome_supervisionrecord treatment race_* gender_* acs_* sd_*, robust cluster(hhid)
gen observed_beta=_b[treatment]

*inflation factor
regress g10 treatment race_* gender_* acs_* sd_*, robust cluster(hhid)
gen observed_inflationfactor=1/(_b[treatment])

forvalues ctr = 1(1)`numberofreps' {
 
 di "Working on replication `ctr' of `numberofreps' at `c(current_time)'"

 * Random number for assigning treatment
 sort hhid
 gen rand_num=runiform()
 * Random number of picking person in HH
 sort hhid
 qui gen rand_in_hh=runiform()

 * If this simulation has already been run, we can skip all the below
 if `ctr'>`completedsims' {
 
	 *randomly assign 90% to treatment in each state
	 sort match_state rand_num
	 bysort match_state: gen rank_in_state=_n
	 qui replace rank_in_state=rank_in_state/num_in_state
	 gen random_treat=(rank_in_state<=.90)
	 
	 *because people are in households, we need to make sure that everyone gets the same value in a household. Randomly pick one person in each household
	 sort hhid rand_in_hh
	 qui replace random_treat=random_treat[_n-1] if hhid==hhid[_n-1]
	 
	 *run regression, store beta
	 qui regress outcome_supervisionrecord random_treat race_* gender_* acs_* sd_*, robust cluster(hhid)
	 qui replace simulated_beta= _b[random_treat] if rep_number==`ctr'
	 	 
	 drop rank_in_state random_treat 
	 
	 * Every 100 reps, save out results file
	 if int(`ctr'/100)==`ctr'/100 {
	  preserve
	  keep simulated_beta observed_beta observed_inflationfactor rep_number
	  keep if simulated_beta ~=.
	  save intermediate_randomization_test_results.dta, replace
	  restore
	  }  
	}

 drop rand_num rand_in_hh 
	
}
 
keep rep_number simulated_beta observed_beta observed_inflationfactor
drop if rep_number == .
save randomization_test_results.dta, replace
!erase intermediate_randomization_test_results.dta

qui summ observed_beta
local estimate=r(mean)
qui count
local N=r(N)
count if simulated_beta>=`estimate'

disp "Randomization inference result: Simulated Beta > Observed beta in (`r(N)'/`N') proportion of `N' trials"
disp (`r(N)'/`N')


